package de.howaner.Poketherus;

import com.badlogic.gdx.Application.ApplicationType;
import com.badlogic.gdx.Game;
import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.Graphics;
import com.badlogic.gdx.math.Vector2;
import de.howaner.Poketherus.multiplayer.ClientHandle;
import de.howaner.Poketherus.multiplayer.NetworkManager;
import de.howaner.Poketherus.resources.TextureManager;
import de.howaner.Poketherus.screens.SplashScreen;

public class PokemonGame extends Game {
	private static PokemonGame instance;
	private final Vector2 oldFullscreenResoultion = new Vector2();
	private ClientHandle multiplayerHandler;
	private long lastTickTime = 0L;

	public PokemonGame() {
		instance = this;
		this.oldFullscreenResoultion.set(1280, 720);
		this.lastTickTime = System.currentTimeMillis();
	}

	public static PokemonGame getGameInstance() {
		return instance;
	}

	public ClientHandle getMultiplayerHandler() {
		return this.multiplayerHandler;
	}

	public void setMultiplayerHandler(ClientHandle handler) {
		this.multiplayerHandler = handler;
	}

	public void tick() {
		if (this.multiplayerHandler != null) {
			this.networkTick();
		}
	}

	private void networkTick() {
		NetworkManager netManager = this.multiplayerHandler.getNetworkManager();
		if (!netManager.isChannelOpen()) {
			if (netManager.getDisconnectMessage().isEmpty()) {
				this.multiplayerHandler.onDisconnect("Disconnected");
			} else {
				this.multiplayerHandler.onDisconnect(netManager.getDisconnectMessage());
			}

			this.multiplayerHandler = null;
		} else {
			try {
				netManager.processQueue();
			} catch (Exception ex) {
				Gdx.app.error("Multiplayer", "Failed to handle packet", ex);
				netManager.closeChannel("Big packet error!");
			}
		}
	}

	@Override
	public void create() {
		Gdx.graphics.setVSync(false);
		this.setScreen(new SplashScreen());
		TextureManager.load();
	}

	@Override
	public void dispose() {
		super.dispose();
		TextureManager.unload();

		if (this.multiplayerHandler != null) {
			Gdx.app.log("Multiplayer", "Disconnected");
			this.multiplayerHandler.getNetworkManager().closeChannel("Client closed");
			this.networkTick();
		}
	}

	@Override
	public void render() {
		super.render();

		// TICK:
		long currentTime = System.currentTimeMillis();
		long pastTime = currentTime - this.lastTickTime;
		if (pastTime > 50L) {
			this.tick();
			this.lastTickTime = currentTime;
		}
	}

	@Override
	public void resize(int width, int height) {
		super.resize(width, height);
	}

	@Override
	public void pause() {
		super.pause();
	}

	@Override
	public void resume() {
		super.resume();
	}

	public void switchFullscreen() {
		if (Gdx.app.getType() == ApplicationType.Android) {
			return;
		}

		if (Gdx.graphics.isFullscreen()) {
			Gdx.graphics.setDisplayMode((int) this.oldFullscreenResoultion.x, (int) this.oldFullscreenResoultion.y, false);
		} else {
			this.oldFullscreenResoultion.set(Gdx.graphics.getWidth(), Gdx.graphics.getHeight());

			Graphics.DisplayMode mode = Gdx.graphics.getDesktopDisplayMode();
			Gdx.graphics.setDisplayMode(mode.width, mode.height, true);
		}
		Gdx.app.log("Main", "Switched fullscreen!");
	}

}
