package de.howaner.Poketherus.entity;

import com.badlogic.gdx.graphics.Camera;
import com.badlogic.gdx.math.Vector2;
import de.howaner.Poketherus.PokemonGame;
import de.howaner.Poketherus.input.GameInput.WalkFace;
import de.howaner.Poketherus.multiplayer.ClientHandle;
import de.howaner.Poketherus.multiplayer.packets.out.PacketOutChangeState;
import de.howaner.Poketherus.multiplayer.packets.out.PacketOutPlayerMove;
import de.howaner.Poketherus.screens.GameScreen;

public class CameraPlayer extends Player {
	private final Vector2 lastChunk = new Vector2();

	public CameraPlayer(int entityID, String name) {
		super(entityID, name);
	}

	public void setCameraPosition(Camera camera) {
		if (this.isInJump()) {
			// Jump position
			camera.position.set(this.getShadowSprite().getX() + this.getShadowSprite().getWidth() / 2, this.getShadowSprite().getY() + this.getHeight() / 2, 0);
			return;
		}

		// Normal position
		camera.position.set(this.getX() + this.getWidth() / 2, this.getY() + this.getHeight() / 2, 0);
	}

	@Override
	public void onFinishedWalk(WalkSource source) {
		super.onFinishedWalk(source);

		if (this.getChunkX() != this.lastChunk.x || this.getChunkY() != this.lastChunk.y) {
			// Refresh chunks
			GameScreen.getGameScreen().updateChunks(true);
			this.updateLastChunk();
		}

		// Next Walk?
		WalkFace nextWalk = GameScreen.getGameScreen().getPlayerInput().getCurrentWalkInput();
		if (nextWalk == null || GameScreen.getGameScreen().getPlayerInput().isBlocked()) {
			// No more walk
			if (this.getState() == EntityState.SPRINT && PokemonGame.getGameInstance().getMultiplayerHandler() != null) {
				PokemonGame.getGameInstance().getMultiplayerHandler().sendPacket(new PacketOutChangeState(EntityState.NORMAL));
			}
			this.setAnimation(0);
		} else {
			// Next walk
			this.doStep(nextWalk.getEntityFace());
		}
	}

	public void updateLastChunk() {
		this.lastChunk.x = this.getChunkX();
		this.lastChunk.y = this.getChunkY();
	}

	@Override
	public void onStartWalk() {
		super.onStartWalk();

		ClientHandle handler = PokemonGame.getGameInstance().getMultiplayerHandler();
		if (handler != null) {
			if (this.getState() == EntityState.SPRINT)
				handler.sendPacket(new PacketOutChangeState(EntityState.SPRINT));
			handler.sendPacket(new PacketOutPlayerMove(this));
		}
	}

}
