package de.howaner.Poketherus.gui;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.Pixmap;
import com.badlogic.gdx.graphics.Texture;
import com.badlogic.gdx.graphics.g2d.BitmapFont;
import com.badlogic.gdx.graphics.g2d.TextureRegion;
import com.badlogic.gdx.math.Vector2;
import com.badlogic.gdx.scenes.scene2d.Stage;
import com.badlogic.gdx.scenes.scene2d.ui.Image;
import com.badlogic.gdx.scenes.scene2d.ui.Label;
import com.badlogic.gdx.scenes.scene2d.ui.Label.LabelStyle;
import com.badlogic.gdx.scenes.scene2d.ui.TextField;
import com.badlogic.gdx.scenes.scene2d.utils.TextureRegionDrawable;
import de.howaner.Poketherus.resources.TextureManager;

public class TextFieldWithLabel {
	private static LabelStyle labelStyle;
	private static TextField.TextFieldStyle textFieldStyle;

	static {
		BitmapFont font = new BitmapFont(Gdx.files.internal("gui/mainmenu/font/font.fnt"), false);
		font.setScale(0.9f);
		Texture transparent = new Texture(1, 1, Pixmap.Format.Alpha);

		labelStyle = new LabelStyle(font, Color.WHITE);
		labelStyle.background = new TextureRegionDrawable(new TextureRegion(transparent));

		textFieldStyle = new TextField.TextFieldStyle();
		textFieldStyle.background = labelStyle.background;
		textFieldStyle.cursor = new TextureRegionDrawable(new TextureRegion(TextureManager.MAIN_MENU_TEXTFIELD_CURSOR));
		textFieldStyle.font = font;
		textFieldStyle.fontColor = Color.WHITE;
		textFieldStyle.selection = textFieldStyle.cursor;
	}

	private Vector2 position;
	private Image image;
	private Label label;
	private TextField textField;
	private int width = 631;
	private int height = 41;
	private float labelWidth;

	public TextFieldWithLabel(float x, float y, String label) {
		this.position = new Vector2(x, y);
		this.labelWidth = labelStyle.font.getBounds(label).width;
		this.setupComponents(label);
	}

	public float getX() {
		return this.position.x;
	}

	public float getY() {
		return this.position.y;
	}

	public Image getImage() {
		return this.image;
	}

	public Label getLabel() {
		return this.label;
	}

	public TextField getTextField() {
		return this.textField;
	}

	public void setLabelWidth(int labelWidth) {
		this.labelWidth = labelWidth;
		this.setPosition(this.getX(), this.getY());
	}

	public void setPosition(float x, float y) {
		this.position.set(x, y);

		this.image.setPosition(x, y);
		this.label.setPosition(x + 10f, y);
		this.textField.setPosition(x + 10f + 10f + this.labelWidth, y);
	}

	public void setCenterPosition(float x, float y) {
		this.image.setCenterPosition(x, y);
		this.setPosition(this.image.getX(), this.image.getY());
	}

	public void addToStage(Stage stage) {
		stage.addActor(this.image);
		stage.addActor(this.label);
		stage.addActor(this.textField);
	}

	private void setupComponents(String label) {
		this.image = new Image(TextureManager.MAIN_MENU_TEXTFIELD);

		this.label = new Label(label, labelStyle);
		this.label.setWidth(this.labelWidth);
		this.label.setHeight(this.height);

		this.textField = new TextField("", textFieldStyle);
		this.textField.setWidth(this.width - 10f - this.labelWidth - 10f);
		this.textField.setHeight(this.height);

		this.setPosition(this.position.x, this.position.y);
	}

}
