package de.howaner.Poketherus.input;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.Input.Buttons;
import com.badlogic.gdx.Input.Keys;
import com.badlogic.gdx.InputProcessor;
import com.badlogic.gdx.math.Vector2;
import de.howaner.Poketherus.PokemonGame;
import de.howaner.Poketherus.entity.EntityFace;
import de.howaner.Poketherus.entity.EntityState;
import de.howaner.Poketherus.screens.GameScreen;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class GameInput implements InputProcessor {
	private final GameScreen screen;
	private final List<Integer> pressedKeys = new ArrayList<Integer>();
	private final Map<Integer, Vector2> touchLocations = new HashMap<Integer, Vector2>();
	private boolean isBlocked = false;

	public GameInput(GameScreen screen) {
		this.screen = screen;
	}

	public boolean isBlocked() {
		return this.isBlocked;
	}

	public void setBlocked(boolean blocked) {
		this.isBlocked = blocked;
	}

	public WalkFace getCurrentWalkInput() {
		WalkFace walkFace = this.getFaceFromWalkInput();
		if (walkFace == null && !this.touchLocations.isEmpty()) {
			walkFace = this.getFaceFromWalkTouch();
		}

		return walkFace;
	}

	public boolean isPressed(int key) {
		return this.pressedKeys.contains(key);
	}

	public WalkFace getFaceFromWalkInput() {
		for (int i = this.pressedKeys.size() - 1; i >= 0; i--) {
			int key = this.pressedKeys.get(i);
			switch (key) {
				case Keys.W: return WalkFace.NORTH;
				case Keys.A: return WalkFace.WEST;
				case Keys.S: return WalkFace.SOUTH;
				case Keys.D: return WalkFace.EAST;
			}
		}
		return null;
	}

	public WalkFace getFaceFromWalkTouch() {
		List<WalkFace> walkFaces = new ArrayList<WalkFace>();
		for (int i = this.touchLocations.size() - 1; i >= 0; i--) {
			Vector2 loc = this.touchLocations.get(i);
			if (loc == null) {
				continue;
			}

			WalkFace walkFace = this.getFaceFromWalkTouch((int)loc.x, (int)loc.y);
			if (walkFace != null) {
				walkFaces.add(walkFace);
			}
		}

		if (walkFaces.isEmpty()) {
			return null;
		}
		WalkFace walkFace = walkFaces.get(0);

		switch (walkFace) {
			case NORTH:
			{
				if (walkFaces.contains(WalkFace.WEST)) {
					return WalkFace.WEST_NORTH;
				} else if (walkFaces.contains(WalkFace.EAST)) {
					return WalkFace.NORTH_EAST;
				} else {
					return WalkFace.NORTH;
				}
			}
			case EAST:
			{
				if (walkFaces.contains(WalkFace.NORTH)) {
					return WalkFace.NORTH_EAST;
				} else if (walkFaces.contains(WalkFace.SOUTH)) {
					return WalkFace.EAST_SOUTH;
				} else {
					return WalkFace.EAST;
				}
			}
			case SOUTH:
			{
				if (walkFaces.contains(WalkFace.EAST)) {
					return WalkFace.EAST_SOUTH;
				} else if (walkFaces.contains(WalkFace.WEST)) {
					return WalkFace.SOUTH_WEST;
				} else {
					return WalkFace.SOUTH;
				}
			}
			case WEST:
			{
				if (walkFaces.contains(WalkFace.SOUTH)) {
					return WalkFace.SOUTH_WEST;
				} else if (walkFaces.contains(WalkFace.NORTH)) {
					return WalkFace.WEST_NORTH;
				} else {
					return WalkFace.WEST;
				}
			}
		}
		return null;
	}

	public WalkFace getFaceFromWalkTouch(int x, int y) {
		int halfX = Gdx.graphics.getWidth() / 2;
		int halfY = Gdx.graphics.getHeight() / 2;
		float scale = (float)Gdx.graphics.getHeight() / (float)Gdx.graphics.getWidth();
		if (scale == 0f) {
			return null;
		}

		x = x - halfX;
		if (y < halfY) {
			// North
			y = halfY - y;
		} else {
			// South
			y = -(y - halfY);
		}

		int xRight = Math.round(y / scale); //g2
		int xLeft = -xRight; //g1

		if (y >= 0) {
			if (x < xLeft) {
				return WalkFace.WEST;
			} else if (x > xRight) {
				return WalkFace.EAST;
			} else {
				return WalkFace.NORTH;
			}
		} else if (y < 0) {
			if (x < xRight) {
				return WalkFace.WEST;
			} else if (x > xLeft) {
				return WalkFace.EAST;
			} else {
				return WalkFace.SOUTH;
			}
		}
		return null;
	}

	@Override
	public boolean keyDown(final int key) {
		if (!this.pressedKeys.contains(key)) {
			this.pressedKeys.add(key);
		}

		if (this.isBlocked) {
			return false;
		}

		if (key == Keys.SHIFT_LEFT) {
			this.screen.getPlayer().setState(EntityState.SPRINT);
			Gdx.app.log("GameInput", "Enabled sprinting!");
		}

		if (!this.screen.getPlayer().canStartWalk()) {
			return false;
		}

		WalkFace walkFace = this.getFaceFromWalkInput();
		if (walkFace != null) {
			EntityFace face = walkFace.getEntityFace();
			this.screen.getPlayer().doStep(face);
			return true;
		}

		switch (key) {
			case Keys.F3: {
				this.screen.getDebugRenderer().setEnabled(!this.screen.getDebugRenderer().isEnabled());
				return true;
			}
			case Keys.F11: {
				PokemonGame.getGameInstance().switchFullscreen();
				return true;
			}
		}
		return false;
	}

	@Override
	public boolean keyUp(int key) {
		if (key == Keys.SHIFT_LEFT) {
			this.screen.getPlayer().setState(EntityState.NORMAL);
			Gdx.app.log("GameInput", "Disabled sprinting!");
		}

		for (int i = 0; i < this.pressedKeys.size(); i++) {
			int pKey = this.pressedKeys.get(i);
			if (pKey == key) {
				this.pressedKeys.remove(i);
				break;
			}
		}
		return true;
	}

	@Override
	public boolean keyTyped(char c) {
		return false;
	}

	@Override
	public boolean touchDown(int screenX, int screenY, int pointer, int button) {
		if (button != Buttons.LEFT) {
			return false;
		}

		this.touchLocations.put(pointer, new Vector2(screenX, screenY));

		if (this.isBlocked || !this.screen.getPlayer().canStartWalk()) {
			return false;
		}

		WalkFace walkFace = this.getFaceFromWalkTouch();
		if (walkFace != null) {
			EntityFace face = walkFace.getEntityFace();
			this.screen.getPlayer().setFaceDirection(face);
			this.screen.getPlayer().doStep(face);

			return true;
		}

		return false;
	}

	@Override
	public boolean touchUp(int screenX, int screenY, int pointer, int button) {
		this.touchLocations.remove(pointer);
		return false;
	}

	@Override
	public boolean touchDragged(int screenX, int screenY, int pointer) {
		if (this.touchLocations.containsKey(pointer)) {
			this.touchLocations.get(pointer).set(screenX, screenY);
		} else {
			this.touchLocations.put(pointer, new Vector2(screenX, screenY));
		}
		return true;
	}

	@Override
	public boolean mouseMoved(int i, int i1) {
		return false;
	}

	@Override
	public boolean scrolled(int i) {
		return false;
	}

	public enum WalkFace {
		NORTH, EAST, SOUTH, WEST, WEST_NORTH, NORTH_EAST, EAST_SOUTH, SOUTH_WEST;

		private int oldState = 1;

		public EntityFace getEntityFace() {
			switch (this) {
				case NORTH: return EntityFace.NORTH;
				case EAST: return EntityFace.EAST;
				case SOUTH: return EntityFace.SOUTH;
				case WEST: return EntityFace.WEST;
				case WEST_NORTH:
				{
					if (this.oldState == 1) {
						this.oldState = 2;
						return EntityFace.WEST;
					} else {
						this.oldState = 1;
						return EntityFace.NORTH;
					}
				}
				case NORTH_EAST:
				{
					if (this.oldState == 1) {
						this.oldState = 2;
						return EntityFace.NORTH;
					} else {
						this.oldState = 1;
						return EntityFace.EAST;
					}
				}
				case EAST_SOUTH:
				{
					if (this.oldState == 1) {
						this.oldState = 2;
						return EntityFace.EAST;
					} else {
						this.oldState = 1;
						return EntityFace.SOUTH;
					}
				}
				case SOUTH_WEST:
				{
					if (this.oldState == 1) {
						this.oldState = 2;
						return EntityFace.SOUTH;
					} else {
						this.oldState = 1;
						return EntityFace.WEST;
					}
				}
			}
			return null;
		}
	}

}