package de.howaner.Poketherus.map;

import com.badlogic.gdx.utils.Disposable;
import de.howaner.Poketherus.map.events.Event;
import de.howaner.Poketherus.map.events.MapEvents;
import de.howaner.Poketherus.saveformat.TagBase;
import de.howaner.Poketherus.saveformat.TagCompound;
import de.howaner.Poketherus.saveformat.TagList;
import java.util.ArrayList;
import java.util.List;

public class MapTerrain implements Disposable {
	private String name;
	private String joinMessage = "";
	private int width;
	private int height;
	private List<MapLayer> layers;
	private MapTileset tileset;
	private MapWalkData walkData;
	private MapEvents events;

	public MapTerrain() {
		this("unknown", 40, 40);
	}

	public MapTerrain(String name, int width, int height) {
		this.name = name;
		this.width = width;
		this.height = height;
		this.layers = new ArrayList<MapLayer>();
		this.tileset = new MapTileset();
		this.walkData = new MapWalkData(this);
		this.events = new MapEvents(this);
	}
	
	public void setName(String name) {
		this.name = name;
	}

	public String getName() {
		return this.name;
	}

	public String getJoinMessage() {
		return this.joinMessage;
	}

	public void setJoinMessage(String joinMessage) {
		this.joinMessage = joinMessage;
	}

	public int getWidth() {
		return this.width;
	}

	public int getHeight() {
		return this.height;
	}

	public int getLayersNum() {
		return this.layers.size();
	}

	public short getTileAt(int x, int y, int layer) {
		if (layer < 0 || layer >= this.layers.size()) {
			return 0;
		}

		return this.layers.get(layer).getTile(x, y);
	}

	public MapWalkData.WalkDataType getWalkdataAt(int x, int y) {
		return this.walkData.getWalkData(x, y);
	}

	public Event getEventAt(int x, int y) {
		return this.events.getEventAt(x, y);
	}

	public MapTileset getTileset() {
		return this.tileset;
	}

	public void setTileset(MapTileset tileset) {
		this.tileset = tileset;
	}
	
	public void setWalkData(MapWalkData walkData) {
		this.walkData = walkData;
	}

	public void setEvents(MapEvents events) {
		this.events = events;
	}
	
	public void loadFromCompound(TagCompound compound) {
		if (compound.containsTag("joinMessage"))
			this.joinMessage = compound.getString("joinMessage");
		this.width = compound.getInt("width");
		this.height = compound.getInt("height");

		// Load tileset
		TagCompound tilesetCompound = compound.getCompound("tileset");
		this.tileset.loadFromCompound(tilesetCompound);

		// Load layers
		List<TagBase> layersList = compound.getList("layers");
		for (TagBase layerTag : layersList) {
			MapLayer layer = new MapLayer(this.layers.size(), this);
			layer.loadFromTagList(((TagList) layerTag).getData());
			this.layers.add(layer);
		}

		// Load walkdata
		this.walkData = new MapWalkData(this);
		this.walkData.loadFromTagList(compound.getList("walkData"));

		// Load entities
		if (compound.containsTag("events")) {
			this.events = new MapEvents(this);
			this.events.loadFromTagList(compound.getList("events"));
		}
	}

	@Override
	public void dispose() {
		this.events.dispose();
		this.tileset.dispose();
		this.walkData.dispose();
		this.joinMessage = "";

		for (int i = 0; i < this.layers.size(); i++) {
			this.layers.get(i).dispose();
		}
		this.layers.clear();
	}

}
