package de.howaner.Poketherus.map;

import com.badlogic.gdx.utils.Disposable;
import de.howaner.Poketherus.entity.EntityFace;
import de.howaner.Poketherus.saveformat.TagBase;
import de.howaner.Poketherus.saveformat.TagShort;
import java.util.List;

public class MapWalkData implements Disposable {
	private int width;
	private int height;
	private WalkDataType[] walkData;

	public MapWalkData(MapTerrain map) {
		this(map.getWidth(), map.getHeight());
	}

	public MapWalkData(int width, int height) {
		this.width = width;
		this.height = height;
		this.walkData = new WalkDataType[width * height];
		this.fillWalkData();
	}

	public int getWidth() {
		return this.width;
	}

	public int getHeight() {
		return this.height;
	}

	private void fillWalkData() {
		for (int i = 0; i < this.walkData.length; i++) {
			this.walkData[i] = WalkDataType.NONE;
		}
	}
	
	public WalkDataType[] getDataArray() {
		return this.walkData;
	}
	
	public WalkDataType getWalkData(int x, int y) {
		if (x < 0 || x >= this.getWidth() || y < 0 || y >= this.getHeight()) {
			return WalkDataType.BLOCKED;
		}
		return this.walkData[y * this.getWidth() + x];
	}

	public void loadFromTagList(List<TagBase> list) {
		this.walkData = new WalkDataType[this.getWidth() * this.getHeight()];
		this.fillWalkData();
		for (int i = 0; i < list.size(); i++) {
			TagShort tag = (TagShort) list.get(i);
			this.walkData[i] = WalkDataType.getFromId(tag.getData());
		}
	}

	@Override
	public void dispose() {
		for (int i = 0; i < this.walkData.length; i++) {
			this.walkData[i] = null;
		}
		this.walkData = null;
	}

	public enum WalkDataType {
		NONE, Layer0, Layer1, Layer2, Layer3, Layer4, ONLY_NORTH, ONLY_EAST, ONLY_SOUTH, ONLY_WEST, BLOCKED, SURFER, DIVE, KASCADE;

		/** Returns the face direction if the walk data is only_xxxx. Otherwise it returns null. */
		public EntityFace getAttachedFace() {
			switch (this) {
				case ONLY_NORTH:
					return EntityFace.NORTH;
				case ONLY_EAST:
					return EntityFace.EAST;
				case ONLY_SOUTH:
					return EntityFace.SOUTH;
				case ONLY_WEST:
					return EntityFace.WEST;
				default:
					return null;
			}
		}

		public static WalkDataType getFromId(int id) {
			if (id < 0 || id >= values().length) {
				return null;
			}

			return values()[id];
		}
	}

}
