package de.howaner.Poketherus.map;

import com.badlogic.gdx.graphics.Pixmap;
import com.badlogic.gdx.graphics.Texture;
import com.badlogic.gdx.graphics.g2d.Gdx2DPixmap;
import com.badlogic.gdx.utils.Disposable;
import de.howaner.Poketherus.saveformat.TagCompound;
import de.howaner.Poketherus.util.Base64;
import java.io.ByteArrayInputStream;
import java.io.InputStream;

public class TileAnimation implements Disposable {
	private TileAnimationType animationType = null;
	private Texture animationImage = null;
	private float animationDuration = 0.10f;
	private float animationDelay = 1f;

	private String particleName = "";
	private float particleDuration = 0.25f;

	private boolean hidePlayerFoot = false;

	public TileAnimationType getAnimationType() {
		return this.animationType;
	}

	public Texture getAnimationImage() {
		return this.animationImage;
	}

	public float getAnimationDuration() {
		return this.animationDuration;
	}

	public float getAnimationDelay() {
		return this.animationDelay;
	}

	public String getParticleName() {
		return this.particleName;
	}

	public float getParticleDuration() {
		return this.particleDuration;
	}

	public boolean getHidePlayerFoot() {
		return this.hidePlayerFoot;
	}

	public void setAnimationType(TileAnimationType type) {
		this.animationType = type;
	}

	public void setAnimationImage(Texture image) {
		this.animationImage = image;
	}

	public void setAnimationDuration(float duration) {
		this.animationDuration = duration;
	}

	public void setAnimationDelay(float delay) {
		this.animationDelay = delay;
	}

	public void setParticleName(String particleName) {
		this.particleName = particleName;
	}

	public void setParticleDuration(float duration) {
		this.particleDuration = duration;
	}

	public void setHidePlayerFoot(boolean hidePlayerFoot) {
		this.hidePlayerFoot = hidePlayerFoot;
	}

	public void loadFromCompound(TagCompound compound) throws Exception {
		if (compound.containsTag("AnimationType")) {
			this.animationType = TileAnimationType.valueOf(compound.getString("AnimationType"));
			this.animationDuration = compound.getFloat("AnimationDuration");
			this.animationDelay = compound.getFloat("AnimationDelay");

			byte[] bytes = Base64.decode(compound.getString("AnimationImage"), Base64.DEFAULT);
			InputStream inputStream = new ByteArrayInputStream(bytes);
			
			Gdx2DPixmap pixmap = new Gdx2DPixmap(inputStream, Gdx2DPixmap.GDX2D_FORMAT_RGBA8888);
			this.animationImage = new Texture(new Pixmap(pixmap));
		}

		if (compound.containsTag("ParticleName")) {
			this.particleName = compound.getString("ParticleName");
			this.particleDuration = compound.getFloat("ParticleDuration");
		}

		if (compound.containsTag("HidePlayerFoot")) {
			this.hidePlayerFoot = compound.getBoolean("HidePlayerFoot");
		}
	}

	@Override
	public void dispose() {
		this.animationImage.dispose();
	}

	public enum TileAnimationType {
		NORMAL, WALK, STAND;
	}

}
