package de.howaner.Poketherus.map.renderer;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.graphics.Texture;
import com.badlogic.gdx.graphics.g2d.Animation;
import com.badlogic.gdx.graphics.g2d.Animation.PlayMode;
import com.badlogic.gdx.graphics.g2d.TextureRegion;
import de.howaner.Poketherus.map.MapTileset;
import de.howaner.Poketherus.map.TileAnimation;

public class AnimatedTilesetTile extends TilesetTile {
	private TextureRegion staticRegion;
	private final TextureRegion[] animationRegions;

	private final TileAnimation animation;
	private final Animation playedAnimation;
	private float animationDelay = 0f;
	private float animationStepTime = 0f;

	public AnimatedTilesetTile(MapTileset tileset, Texture staticTexture, TileAnimation animation, short tileId) {
		super(tileset, tileId);
		this.staticRegion = new TextureRegion(staticTexture);
		this.animation = animation;

		this.animationRegions = new TextureRegion(animation.getAnimationImage()).split(16, 16)[0];
		this.animationDelay = animation.getAnimationDelay();
		this.animationStepTime = 0f;

		this.playedAnimation = new Animation(animation.getAnimationDuration(), this.animationRegions);
		this.playedAnimation.setPlayMode(PlayMode.NORMAL);
	}

	@Override
	public TextureRegion getTextureRegion(int tileX, int tileY) {
		if (this.playedAnimation.isAnimationFinished(this.animationStepTime)) {
			this.animationDelay -= Gdx.graphics.getDeltaTime();
			if (this.animationDelay <= 0f) {
				this.animationStepTime = 0f;
				this.animationDelay = this.animation.getAnimationDelay();
			}
			return this.staticRegion;
		}

		TextureRegion region = this.playedAnimation.getKeyFrame(this.animationStepTime);
		this.animationStepTime += Gdx.graphics.getDeltaTime();
		return region;
	}

}
