package de.howaner.Poketherus.map.renderer;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.graphics.Texture;
import com.badlogic.gdx.graphics.g2d.Animation;
import com.badlogic.gdx.graphics.g2d.Animation.PlayMode;
import com.badlogic.gdx.graphics.g2d.TextureRegion;
import de.howaner.Poketherus.map.MapTileset;
import de.howaner.Poketherus.map.TileAnimation;
import de.howaner.Poketherus.map.TileAnimation.TileAnimationType;
import java.util.HashMap;
import java.util.Map;

public class WalkAnimatedTilesetTile extends TilesetTile {
	private final TextureRegion staticRegion;
	private final TextureRegion[] animationRegions;

	private final TileAnimation animation;
	private final Map<String, AnimatedCoord> playedAnimations = new HashMap<String, AnimatedCoord>();

	public WalkAnimatedTilesetTile(MapTileset tileset, Texture staticTexture, TileAnimation animation, short tileId) {
		super(tileset, tileId);
		this.staticRegion = new TextureRegion(staticTexture);

		this.animation = animation;
		this.animationRegions = new TextureRegion(this.animation.getAnimationImage()).split(16, 16)[0];
	}

	public void play(int tileX, int tileY) {
		AnimatedCoord coord = new AnimatedCoord();
		coord.playedAnimation = new Animation(this.animation.getAnimationDuration(), animationRegions);
		coord.playedAnimation.setPlayMode(PlayMode.NORMAL);
		coord.playedAnimationStepTime = 0f;

		this.playedAnimations.put(this.toString(tileX, tileY), coord);
	}

	public void stopAnimation(int tileX, int tileY) {
		this.playedAnimations.remove(this.toString(tileX, tileY));
	}

	@Override
	public TextureRegion getTextureRegion(int tileX, int tileY) {
		AnimatedCoord animation = this.playedAnimations.get(this.toString(tileX, tileY));
		if (animation != null) {
			TextureRegion region = animation.playedAnimation.getKeyFrame(animation.playedAnimationStepTime);
			animation.playedAnimationStepTime += Gdx.graphics.getDeltaTime();

			if (this.animation.getAnimationType() != TileAnimationType.STAND && animation.playedAnimation.isAnimationFinished(animation.playedAnimationStepTime)) {
				this.stopAnimation(tileX, tileY);
			}
			return region;
		}

		return this.staticRegion;
	}

	public TileAnimation getAnimation() {
		return this.animation;
	}

	public boolean isPlaying(int tileX, int tileY) {
		AnimatedCoord animation = this.playedAnimations.get(this.toString(tileX, tileY));
		return (animation != null && !animation.playedAnimation.isAnimationFinished(animation.playedAnimationStepTime));
	}

	private String toString(int tileX, int tileY) {
		return String.format("%d:%d", tileX, tileY);
	}

	private class AnimatedCoord {
		public Animation playedAnimation = null;
		public float playedAnimationStepTime = 0f;
	}

}
