package de.howaner.Poketherus.map.renderer.particle;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.graphics.g2d.Animation;
import com.badlogic.gdx.graphics.g2d.Batch;
import com.badlogic.gdx.graphics.g2d.TextureAtlas;
import com.badlogic.gdx.graphics.g2d.TextureAtlas.AtlasRegion;
import com.badlogic.gdx.utils.Array;
import com.badlogic.gdx.utils.Disposable;
import java.util.ArrayList;
import java.util.List;

public class ParticleRenderer implements Disposable {
	private TextureAtlas particleMap;
	private final List<Particle> currentParticles = new ArrayList<Particle>();

	public ParticleRenderer() {
		try {
			this.particleMap = new TextureAtlas("textures/particles.pack");
		} catch (Exception ex) {
			Gdx.app.error("ParticleRenderer", "Can't load particle map", ex);
			this.particleMap = new TextureAtlas();
		}
	}

	public void renderParticles(Batch batch) {
		List<Particle> particlesToRemove = new ArrayList<Particle>();
		for (Particle particle : this.currentParticles) {
			if (particle.isFinished()) {
				particlesToRemove.add(particle);
				continue;
			}
			particle.draw(batch);
		}

		// Remove old particles
		for (Particle particle : particlesToRemove) {
			this.currentParticles.remove(particle);
		}
	}

	public void displayParticle(float posX, float posY, float durationInSeconds, String name) {
		Array<AtlasRegion> particleTextures = this.particleMap.findRegions(name);
		if (particleTextures.size == 0) {
			return;
		}

		Animation animation = new Animation(durationInSeconds, particleTextures);
		Particle particle = new Particle(animation);
		particle.setPosition(posX, posY);

		this.currentParticles.add(particle);
	}

	@Override
	public void dispose() {
		this.currentParticles.clear();
		this.particleMap.dispose();
	}

}
