package de.howaner.Poketherus.multiplayer;

import com.badlogic.gdx.Gdx;
import de.howaner.Poketherus.PokemonGame;
import de.howaner.Poketherus.entity.CameraPlayer;
import de.howaner.Poketherus.entity.Entity;
import de.howaner.Poketherus.entity.Player;
import de.howaner.Poketherus.multiplayer.packets.in.*;
import de.howaner.Poketherus.multiplayer.packets.out.*;
import de.howaner.Poketherus.multiplayer.util.CryptManager;
import de.howaner.Poketherus.screens.ConnectScreen;
import de.howaner.Poketherus.screens.DisconnectScreen;
import de.howaner.Poketherus.screens.GameScreen;
import de.howaner.Poketherus.world.World;
import io.netty.util.concurrent.Future;
import io.netty.util.concurrent.GenericFutureListener;
import java.security.PublicKey;
import javax.crypto.Cipher;
import javax.crypto.SecretKey;

public class ClientHandle {
	private final NetworkManager netManager;
	private String ipAddress = null;
	private ConnectScreen connectScreen;

	// Login details:
	private String username, password;

	public ClientHandle(NetworkManager netManager) {
		this.netManager = netManager;
	}

	public ConnectScreen getConnectScreen() {
		return this.connectScreen;
	}

	public void setConnectScreen(ConnectScreen screen) {
		this.connectScreen = screen;
	}

	public NetworkManager getNetworkManager() {
		return this.netManager;
	}

	public String getIPAddress() {
		return this.ipAddress;
	}

	public String getUsername() {
		return this.username;
	}

	public String getPassword() {
		return this.password;
	}

	public void setLoginDetails(String username, String password) {
		this.username = username;
		this.password = password;
	}

	public void connectToServer() {
		this.netManager.addOutboundPacketToQueue(new PacketOutLoginStart((byte) 1, this.username));

		GenericFutureListener sentPacketListener = new GenericFutureListener() {
			@Override
			public void operationComplete(Future f) throws Exception {
				try {
					if (ClientHandle.this.connectScreen != null) {
						ClientHandle.this.connectScreen.setCurrentText("Verschlüsslung aktiviert");
					}

					ClientHandle.this.netManager.enableEncryption(ClientHandle.this.netManager.getSecretKey());
				} catch (Exception ex) {
					if (ClientHandle.this.connectScreen != null) {
						ClientHandle.this.connectScreen.setCurrentText("Die Verschlüsslung konnte nicht aktiviert werden!\n" + ex.getMessage());
					}
					Gdx.app.error("Multiplayer", "Can't enable encryption", ex);
				}
			}
		};

		try {
			PublicKey publicKey = CryptManager.readPublicKeyFromFile(Gdx.files.internal("server.key"));
			SecretKey key = this.netManager.getSecretKey();

			Cipher cipher = Cipher.getInstance("RSA/ECB/PKCS1Padding");
			cipher.init(Cipher.ENCRYPT_MODE, publicKey);
			byte[] encryptedSecretKey = cipher.doFinal(key.getEncoded());

			this.netManager.addOutboundPacketToQueue(new PacketOutEncryptionKey(encryptedSecretKey), sentPacketListener);
		} catch (Exception ex) {
			Gdx.app.error("Multiplayer", "Can't send encryption key", ex);
		}
	}
	
	public void onNetworkTick() {
		
	}

	public void onDisconnect(String message) {
		Gdx.app.log("Multiplayer", "Disconnected: " + message);
		PokemonGame.getGameInstance().setScreen(new DisconnectScreen(message));
	}

	public void sendPacket(OutPacket packet) {
		this.sendPacket(packet, new GenericFutureListener[0]);
	}

	public void sendPacket(OutPacket packet, GenericFutureListener[] listeners) {
		try {
			this.netManager.scheduleOutboundPacket(packet, listeners);
		} catch (Exception ex) {
			Gdx.app.error("Multiplayer", "Can't send packet", ex);
		}
	}

	///---- PACKET HANDLERS ----///
	public void handlePacketKeepAlive(PacketInKeepAlive packet) {
		this.sendPacket(new PacketOutKeepAlive(packet.getTime()));
	}

	public void handlePacketDisconnect(PacketInDisconnect packet) {
		this.netManager.closeChannel(packet.getMessage());
	}

	public void handlePacketAuthenticationRequest(PacketInAuthenticationRequest packet) {
		if (packet.getIPAddress().isEmpty()) {
			this.netManager.closeChannel("Empty ip address??");
			return;
		}

		this.ipAddress = packet.getIPAddress();
		String passwordHash = CryptManager.createMD5(CryptManager.createMD5(this.password) + packet.getIPAddress());
		this.sendPacket(new PacketOutAuthenticationResponse(passwordHash));
	}

	public void handlePacketLoginSuccess(PacketInLoginSuccess packet) {
		if (this.connectScreen != null) {
			this.connectScreen.setCurrentText("Login erfolgreich");
		}
		Gdx.app.log("Multiplayer", "Connected to server!");

		CameraPlayer player = new CameraPlayer(packet.getEntityID(), "Olaf");
		player.setPosition(packet.getLocation().getPosX() * 16, packet.getLocation().getPosY() * 16);

		GameScreen gameScreen = new GameScreen(player);
		gameScreen.loadMainWorld();
		gameScreen.switchToWorld(packet.getLocation().getWorld());
		gameScreen.updateChunks(false);
		PokemonGame.getGameInstance().setScreen(gameScreen);
	}

	public void handlePacketPlayerSpawn(PacketInPlayerSpawn packet) {
		Gdx.app.log("Multiplayer", "Spawn player with id " + packet.getEntityID() + " and name " + packet.getUserName());

		Player player = new Player(packet.getEntityID(), packet.getUserName());
		player.setPosition(packet.getLocation().getPosX() * 16, packet.getLocation().getPosY() * 16);
		player.setFaceDirection(packet.getFace());

		GameScreen.getGameScreen().getCurrentWorld().spawnEntity(player);
	}

	public void handlePacketEntityRemove(PacketInEntityRemove packet) {
		Gdx.app.log("Multiplayer", "Removed entity with id " + packet.getEntityID());
		GameScreen.getGameScreen().getCurrentWorld().removeEntity(packet.getEntityID());
	}

	public void handlePacketEntityTeleport(PacketInEntityTeleport packet) {
		Entity entity = GameScreen.getGameScreen().getCurrentWorld().getEntity(packet.getEntityID());
		if (entity == null) {
			return;
		}

		entity.cancelWalk(false);
		entity.setPosition(packet.getDestinition().getPosX() * 16, packet.getDestinition().getPosY() * 16);
		entity.setFaceDirection(packet.getFace());
	}

	public void handlePacketEntityWalk(PacketInEntityWalk packet) {
		Entity entity = GameScreen.getGameScreen().getCurrentWorld().getEntity(packet.getEntityID());
		if (entity == null) {
			return;
		}

		entity.finishWalk();
		if (!entity.getLocation().equalsCoords(packet.getCurrentLocation())) {
			entity.setPosition(packet.getCurrentLocation().getPosX() * 16, packet.getCurrentLocation().getPosY() * 16);
		}

		entity.doStep(packet.getFace());
	}

	public void handlePacketFadeScreen(PacketInFadeScreen packet) {
		if (packet.getFadeType() == PacketInFadeScreen.FadeType.FADE_OUT) {
			this.netManager.setReceivePacketsEnabled(false);
			GameScreen.getGameScreen().getPlayerInput().setBlocked(true);
			GameScreen.getGameScreen().fadeOut(packet.getDuration(), new Runnable() {
				@Override
				public void run() {
					ClientHandle.this.netManager.setReceivePacketsEnabled(true);
				}
			});
		} else {
			GameScreen.getGameScreen().fadeIn(packet.getDuration(), null);
			GameScreen.getGameScreen().getPlayerInput().setBlocked(false);
		}
	}

	public void handlePacketFadeEntity(PacketInFadeEntity packet) {
		if (packet.getFadeType() == PacketInFadeEntity.FadeType.FADE_OUT) {
			GameScreen.getGameScreen().fadeEntityOut(packet.getEntityID(), packet.getDuration());
		} else {
			GameScreen.getGameScreen().fadeEntityIn(packet.getEntityID(), packet.getDuration());
		}
	}

	public void handlePacketSwitchWorld(PacketInSwitchWorld packet) {
		if (!GameScreen.getGameScreen().switchToWorld(packet.getDestinition().getWorld())) {
			this.netManager.closeChannel("Can't switch world?? Come on, report this :)");
			return;
		}

		GameScreen.getGameScreen().getPlayer().cancelWalk(false);
		GameScreen.getGameScreen().getPlayer().setPosition(packet.getDestinition().getPosX() * 16, packet.getDestinition().getPosY() * 16);
	}

	public void handlePacketUpdateEntityID(PacketInUpdateEntityID packet) {
		World world = GameScreen.getGameScreen().getCurrentWorld();
		Player player = GameScreen.getGameScreen().getPlayer();

		world.removeEntity(player.getEntityID());
		player.setEntityID(packet.getNewEntityID());
		world.spawnEntity(player);
		Gdx.app.log("Multiplayer", "Updated entity id to " + packet.getNewEntityID());
	}

	public void handlePacketEntityState(PacketInEntityState packet) {
		Entity entity = GameScreen.getGameScreen().getCurrentWorld().getEntity(packet.getEntityID());
		if (entity == null) {
			return;
		}

		entity.setState(packet.getState());
	}

}
