package de.howaner.Poketherus.multiplayer;

import com.badlogic.gdx.Gdx;
import com.google.common.collect.Queues;
import com.google.common.util.concurrent.ThreadFactoryBuilder;
import de.howaner.Poketherus.multiplayer.netty.MessageDeserializer;
import de.howaner.Poketherus.multiplayer.netty.MessageLengthDeserializer;
import de.howaner.Poketherus.multiplayer.netty.MessageLengthSerializer;
import de.howaner.Poketherus.multiplayer.netty.MessageSerializer;
import de.howaner.Poketherus.multiplayer.netty.NettyEncryptingDecoder;
import de.howaner.Poketherus.multiplayer.netty.NettyEncryptingEncoder;
import de.howaner.Poketherus.multiplayer.packets.in.InPacket;
import de.howaner.Poketherus.multiplayer.packets.out.OutPacket;
import de.howaner.Poketherus.multiplayer.util.CryptManager;
import io.netty.bootstrap.Bootstrap;
import io.netty.channel.Channel;
import io.netty.channel.ChannelException;
import io.netty.channel.ChannelFutureListener;
import io.netty.channel.ChannelHandlerContext;
import io.netty.channel.ChannelInitializer;
import io.netty.channel.ChannelOption;
import io.netty.channel.SimpleChannelInboundHandler;
import io.netty.channel.nio.NioEventLoopGroup;
import io.netty.channel.socket.nio.NioSocketChannel;
import io.netty.handler.timeout.ReadTimeoutHandler;
import io.netty.util.concurrent.GenericFutureListener;
import java.net.InetAddress;
import java.net.SocketAddress;
import java.util.Queue;
import java.util.concurrent.TimeoutException;
import javax.crypto.SecretKey;

public class NetworkManager extends SimpleChannelInboundHandler {
	public static final NioEventLoopGroup eventLoops = new NioEventLoopGroup(0, (new ThreadFactoryBuilder()).setNameFormat("Netty Client IO #%d").setDaemon(true).build());

	private final Queue<InPacket> inPacketsQueue = Queues.newConcurrentLinkedQueue();
	private final Queue<QueuedOutPacket> outPacketsQueue = Queues.newConcurrentLinkedQueue();

	/**The active channel */
	private Channel channel;

	/** The address of the remote party */
	private SocketAddress socketAddress;

	/** The client handle */
	private ClientHandle clientHandle;

	/** A String indicating why the network has shutdown. */
	private String disconnectMessage = "";

	private SecretKey secretKey;
	private boolean receivePackets = true;

	public static NetworkManager createNetworkManager(InetAddress address, int port) {
		final NetworkManager manager = new NetworkManager();

		new Bootstrap().group(eventLoops).handler(new ChannelInitializer() {
			@Override
			protected void initChannel(Channel channel) throws Exception {
				try {
					channel.config().setOption(ChannelOption.IP_TOS, 24);
				} catch (ChannelException ex) {}

				try {
					channel.config().setOption(ChannelOption.TCP_NODELAY, false);
				} catch (ChannelException ex) {}

				channel.pipeline().addLast("timeout", new ReadTimeoutHandler(20));
				channel.pipeline().addLast("splitter", new MessageLengthDeserializer());
				channel.pipeline().addLast("decoder", new MessageDeserializer());
				channel.pipeline().addLast("prepender", new MessageLengthSerializer());
				channel.pipeline().addLast("encoder", new MessageSerializer());
				channel.pipeline().addLast("packet_handler", manager);
			}
		}).channel(NioSocketChannel.class).connect(address, port).syncUninterruptibly();

		return manager;
	}

	public NetworkManager() {
		this.clientHandle = new ClientHandle(this);
		try {
			this.secretKey = CryptManager.createSecretKey();
			Gdx.app.log("KeyGen", "Generated private and public key.");
		} catch (Exception ex) {
			Gdx.app.error("KeyGen", "Can't create client private and public key", ex);
		}
	}

	public boolean isReceivePacketsEnabled() {
		return this.receivePackets;
	}

	public void setReceivePacketsEnabled(boolean enabled) {
		this.receivePackets = enabled;
	}

	public SecretKey getSecretKey() {
		return this.secretKey;
	}

	public ClientHandle getClientHandle() {
		return this.clientHandle;
	}

	public boolean isChannelOpen() {
		return (this.channel != null && this.channel.isOpen());
	}

	public String getDisconnectMessage() {
		return this.disconnectMessage;
	}

	public SocketAddress getRemoteAddress() {
		return this.socketAddress;
	}

	public void disableAutoRead() {
		this.channel.config().setAutoRead(false);
	}

	@Override
	public void channelActive(ChannelHandlerContext ctx) throws Exception {
		super.channelActive(ctx);
		this.channel = ctx.channel();
		this.socketAddress = ctx.channel().remoteAddress();
		this.channel.config().setAutoRead(true);
	}

	@Override
	public void channelInactive(ChannelHandlerContext ctx) {
		this.closeChannel("Ende der Datenübertragung oder so");
	}

	@Override
	public void exceptionCaught(ChannelHandlerContext ctx, Throwable cause) {
		String kickMessage;

		if (cause instanceof TimeoutException) {
			kickMessage = "Du bischd ausgetimd! Kauf besseres Internet!";
		} else {
			kickMessage = "Internal Exception: " + cause;
		}

		this.closeChannel(kickMessage);
	}

	/** Closes the channel, the parameter can be used for an exit message (not certain how it gets sent) */
	public void closeChannel(String kickMessage) {
		if (this.channel.isOpen()) {
			this.disconnectMessage = kickMessage;
			this.channel.close();
		}
	}

	@Override
	protected void channelRead0(ChannelHandlerContext ctx, Object msg) throws Exception {
		if (!this.channel.isOpen()) {
			return;
		}
		InPacket packet = (InPacket) msg;

		if (packet.hasPriority()) {
			packet.handlePacket(this.clientHandle);
		} else {
			this.inPacketsQueue.add(packet);
		}
	}

	/** Called every tick */
	public void processQueue() {
		this.flushOutboundQueue();

		int currentPacket = 0;
		while (this.receivePackets && !this.inPacketsQueue.isEmpty() && currentPacket < 1000) {
			InPacket packet = this.inPacketsQueue.poll();
			packet.handlePacket(this.clientHandle);

			currentPacket++;
		}

		this.clientHandle.onNetworkTick();
		this.channel.flush();
	}

	private void flushOutboundQueue() {
		if (this.channel != null && this.channel.isOpen()) {
			while (!this.outPacketsQueue.isEmpty()) {
				QueuedOutPacket queuedPacket = this.outPacketsQueue.poll();
				this.dispatchPacket(queuedPacket.packet, queuedPacket.listeners);
			}
		}
	}

	public void scheduleOutboundPacket(OutPacket packet, GenericFutureListener ... listeners) {
		if (this.channel != null && this.channel.isOpen()) {
			this.dispatchPacket(packet, listeners);
		} else {
			this.outPacketsQueue.add(new QueuedOutPacket(packet, listeners));
		}
	}

	public void addOutboundPacketToQueue(OutPacket packet, GenericFutureListener ... listeners) {
		this.outPacketsQueue.add(new QueuedOutPacket(packet, listeners));
	}

	private void dispatchPacket(final OutPacket packet, final GenericFutureListener[] listeners) {
		if (this.channel.eventLoop().inEventLoop()) {
			// Sync sending
			this.channel.writeAndFlush(packet).addListeners(listeners).addListener(ChannelFutureListener.FIRE_EXCEPTION_ON_FAILURE);
		} else {
			// Async sending
			this.channel.eventLoop().execute(new Runnable() {
				@Override
				public void run() {
					NetworkManager.this.channel.writeAndFlush(packet).addListeners(listeners).addListener(ChannelFutureListener.FIRE_EXCEPTION_ON_FAILURE);
				}
			});
		}
	}

	public void enableEncryption(SecretKey key) throws Exception {
		this.channel.pipeline().addBefore("splitter", "decrypt", new NettyEncryptingDecoder(CryptManager.createDecryptCipher(key)));
		this.channel.pipeline().addBefore("prepender", "encrypt", new NettyEncryptingEncoder(CryptManager.createEncryptCipher(key)));
	}

	public class QueuedOutPacket {
		public final OutPacket packet;
		public final GenericFutureListener[] listeners;

		public QueuedOutPacket(OutPacket packet, GenericFutureListener[] listeners) {
			this.packet = packet;
			this.listeners = listeners;
		}
	}

}
