package de.howaner.Poketherus.multiplayer.util;

import com.badlogic.gdx.files.FileHandle;
import com.google.common.base.Charsets;
import com.google.common.hash.Hashing;
import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.math.BigInteger;
import java.security.KeyFactory;
import java.security.PrivateKey;
import java.security.PublicKey;
import java.security.SecureRandom;
import java.security.spec.EncodedKeySpec;
import java.security.spec.PKCS8EncodedKeySpec;
import java.security.spec.RSAPrivateKeySpec;
import java.security.spec.RSAPublicKeySpec;
import java.security.spec.X509EncodedKeySpec;
import javax.crypto.Cipher;
import javax.crypto.KeyGenerator;
import javax.crypto.SecretKey;
import javax.crypto.spec.IvParameterSpec;

public class CryptManager {
	
	public static PublicKey readPublicKeyFromFile(FileHandle file) throws Exception {
		ObjectInputStream in = new ObjectInputStream(new BufferedInputStream(file.read()));
		try {
			BigInteger m = (BigInteger) in.readObject();
			BigInteger e = (BigInteger) in.readObject();

			RSAPublicKeySpec keySpec = new RSAPublicKeySpec(m, e);
			KeyFactory fact = KeyFactory.getInstance("RSA");
			return fact.generatePublic(keySpec);
		} finally {
			in.close();
		}
	}

	public static PrivateKey readPrivateKeyFromFile(File file) throws Exception {
		ObjectInputStream in = new ObjectInputStream(new BufferedInputStream(new FileInputStream(file)));
		try {
			BigInteger m = (BigInteger) in.readObject();
			BigInteger e = (BigInteger) in.readObject();

			RSAPrivateKeySpec keySpec = new RSAPrivateKeySpec(m, e);
			KeyFactory fact = KeyFactory.getInstance("RSA");
			return fact.generatePrivate(keySpec);
		} finally {
			in.close();
		}
	}

	/** Cipher zum verschlüsseln */
	public static Cipher createEncryptCipher(SecretKey key) throws Exception {
		Cipher cipher = Cipher.getInstance("AES/CFB8/NoPadding");
		cipher.init(Cipher.ENCRYPT_MODE, key, new IvParameterSpec(key.getEncoded()));

		return cipher;
	}

	/** Cipher zum entschlüsseln */
	public static Cipher createDecryptCipher(SecretKey key) throws Exception {
		Cipher cipher = Cipher.getInstance("AES/CFB8/NoPadding");
		cipher.init(Cipher.DECRYPT_MODE, key, new IvParameterSpec(key.getEncoded()));

		return cipher;
	}

	public static SecretKey createSecretKey() throws Exception {
		KeyGenerator keyGen = KeyGenerator.getInstance("AES");
		keyGen.init(128, new SecureRandom());

		return keyGen.generateKey();
	}

	public static void savePrivateKey(PrivateKey privateKey, File file) throws Exception {
		KeyFactory fact = KeyFactory.getInstance("RSA");
		RSAPrivateKeySpec keySpec = fact.getKeySpec(privateKey, RSAPrivateKeySpec.class);

		ObjectOutputStream out = new ObjectOutputStream(new BufferedOutputStream(new FileOutputStream(file)));
		try {
			out.writeObject(keySpec.getModulus());
			out.writeObject(keySpec.getPrivateExponent());
			out.flush();
		} finally {
			out.close();
		}
	}

	public static void savePublicKey(PublicKey publicKey, File file) throws Exception {
		KeyFactory fact = KeyFactory.getInstance("RSA");
		RSAPublicKeySpec keySpec = fact.getKeySpec(publicKey, RSAPublicKeySpec.class);

		ObjectOutputStream out = new ObjectOutputStream(new BufferedOutputStream(new FileOutputStream(file)));
		try {
			out.writeObject(keySpec.getModulus());
			out.writeObject(keySpec.getPublicExponent());
			out.flush();
		} finally {
			out.close();
		}
	}

	public static PrivateKey toPrivateKey(byte[] data) throws Exception {
		KeyFactory fact = KeyFactory.getInstance("RSA");
		EncodedKeySpec privateKeySpec = new PKCS8EncodedKeySpec(data);
		return fact.generatePrivate(privateKeySpec);
	}

	public static PublicKey toPublicKey(byte[] data) throws Exception {
		KeyFactory fact = KeyFactory.getInstance("RSA");
		EncodedKeySpec publicKeySpec = new X509EncodedKeySpec(data);
		return fact.generatePublic(publicKeySpec);
	}

	public static String createMD5(String obj) {
		return Hashing.md5().hashString(obj, Charsets.UTF_8).toString();
	}

}
