package de.howaner.Poketherus.saveformat;

import com.badlogic.gdx.Gdx;
import java.io.DataInput;
import java.io.DataOutput;
import java.io.IOException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;

public class TagCompound extends TagBase {

	private Map<String, TagBase> tags = new HashMap<String, TagBase>();

	public static final Class<?>[] tagTypes = new Class<?>[11];

	static {
		tagTypes[0] = null;
		tagTypes[1] = TagCompound.class;
		tagTypes[2] = TagByte.class;
		tagTypes[3] = TagInt.class;
		tagTypes[4] = TagLong.class;
		tagTypes[5] = TagFloat.class;
		tagTypes[6] = TagDouble.class;
		tagTypes[7] = TagString.class;
		tagTypes[8] = TagList.class;
		tagTypes[9] = TagByteArray.class;
		tagTypes[10] = TagShort.class;
	}

	public Set<String> getKeys() {
		return this.tags.keySet();
	}

	public TagCompound getCompound(String tagName) {
		TagBase tag = this.tags.get(tagName);
		if (tag == null || !(tag instanceof TagCompound)) {
			return null;
		}

		return (TagCompound) tag;
	}

	public Byte getByte(String tagName) {
		TagBase tag = this.tags.get(tagName);
		if (tag == null || !(tag instanceof TagByte)) {
			return null;
		}

		return ((TagByte) tag).getData();
	}

	public Boolean getBoolean(String tagName) {
		Byte b = this.getByte(tagName);
		if (b == null) {
			return null;
		}

		return (b == 1);
	}

	public Integer getInt(String tagName) {
		TagBase tag = this.tags.get(tagName);
		if (tag == null || !(tag instanceof TagInt)) {
			return null;
		}

		return ((TagInt) tag).getData();
	}

	public Long getLong(String tagName) {
		TagBase tag = this.tags.get(tagName);
		if (tag == null || !(tag instanceof TagLong)) {
			return null;
		}

		return ((TagLong) tag).getData();
	}

	public Float getFloat(String tagName) {
		TagBase tag = this.tags.get(tagName);
		if (tag == null || !(tag instanceof TagFloat)) {
			return null;
		}

		return ((TagFloat) tag).getData();
	}

	public Double getDouble(String tagName) {
		TagBase tag = this.tags.get(tagName);
		if (tag == null || !(tag instanceof TagDouble)) {
			return null;
		}

		return ((TagDouble) tag).getData();
	}

	public String getString(String tagName) {
		TagBase tag = this.tags.get(tagName);
		if (tag == null || !(tag instanceof TagString)) {
			return null;
		}

		return ((TagString) tag).getData();
	}

	public List<TagBase> getList(String tagName) {
		TagBase tag = this.tags.get(tagName);
		if (tag == null || !(tag instanceof TagList)) {
			return null;
		}

		return ((TagList) tag).getData();
	}

	public byte[] getByteArray(String tagName) {
		TagBase tag = this.tags.get(tagName);
		if (tag == null || !(tag instanceof TagByteArray)) {
			return null;
		}

		return ((TagByteArray) tag).getData();
	}

	public Short getShort(String tagName) {
		TagBase tag = this.tags.get(tagName);
		if (tag == null || !(tag instanceof TagShort)) {
			return null;
		}

		return ((TagShort) tag).getData();
	}

	public TagBase getTag(String tagName) {
		return this.tags.get(tagName);
	}

	public boolean containsTag(String tagName) {
		return this.tags.containsKey(tagName);
	}

	public void removeTag(String tagName) {
		this.tags.remove(tagName);
	}

	public void setTag(String tagName, TagBase tag) {
		this.tags.put(tagName, tag);
	}

	public void setCompound(String tagName, TagCompound compound) {
		this.tags.put(tagName, compound);
	}

	public void setByte(String tagName, byte data) {
		this.tags.put(tagName, new TagByte(data));
	}

	public void setBoolean(String tagName, boolean data) {
		this.setByte(tagName, (byte) (data ? 1 : 0));
	}
	
	public void setInt(String tagName, int data) {
		this.tags.put(tagName, new TagInt(data));
	}
	
	public void setLong(String tagName, long data) {
		this.tags.put(tagName, new TagLong(data));
	}
	
	public void setFloat(String tagName, float data) {
		this.tags.put(tagName, new TagFloat(data));
	}
	
	public void setDouble(String tagName, double data) {
		this.tags.put(tagName, new TagDouble(data));
	}
	
	public void setString(String tagName, String data) {
		this.tags.put(tagName, new TagString(data));
	}

	public void setList(String tagName, int tagType, List<TagBase> list) {
		this.tags.put(tagName, new TagList(tagType, list));
	}

	public void setByteArray(String tagName, byte[] data) {
		this.tags.put(tagName, new TagByteArray(data));
	}

	public void setShort(String tagName, short data) {
		this.tags.put(tagName, new TagShort(data));
	}

	@Override
	public void read(DataInput input) throws IOException {
		this.tags = new HashMap<String, TagBase>();

		while (true) {
			byte tagType = input.readByte();
			if (tagType == 0) {
				// Finished reading
				break;
			}

			if (tagType < 0 || tagType >= tagTypes.length) {
				// Bad tag type
				break;
			}

			String tagName = input.readUTF();
			Class tagClass = tagTypes[tagType];
			try {
				TagBase tag = (TagBase) tagClass.newInstance();
				tag.read(input);
				this.tags.put(tagName, tag);
			} catch (Exception e) {
				Gdx.app.error("Main", "Can't read nbt tag (type " + tagType + ")", e);
			}
		}
	}

	@Override
	public void write(DataOutput output) throws IOException {
		for (Entry<String, TagBase> e : this.tags.entrySet()) {
			String tagName = e.getKey();
			TagBase tag = e.getValue();

			output.writeByte(tag.getTypeID());
			output.writeUTF(tagName);
			tag.write(output);
		}

		// Write the end "0"
		output.writeByte(0);
	}
}
