package de.howaner.Poketherus.saveformat;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.files.FileHandle;
import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.zip.GZIPInputStream;
import java.util.zip.GZIPOutputStream;

public class TagManager {

	public static TagCompound readCompressed(InputStream inputStream) throws IOException {
		DataInputStream dataInputStream = new DataInputStream(new BufferedInputStream(new GZIPInputStream(inputStream)));
		TagCompound compound = new TagCompound();

		try {
			compound.read(dataInputStream);
		} finally {
			dataInputStream.close();
		}

		return compound;
	}

	public static void writeCompressed(OutputStream outputStream, TagCompound compound) throws IOException {
		DataOutputStream dataOutputStream = new DataOutputStream(new BufferedOutputStream(new GZIPOutputStream(outputStream)));

		try {
			compound.write(dataOutputStream);
			dataOutputStream.flush();
		} finally {
			dataOutputStream.close();
		}
	}

	public static TagCompound readFromFile(File file) throws Exception {
		if (!file.exists()) {
			return null;
		}

		FileInputStream inputStream = new FileInputStream(file);
		TagCompound rootTag = readCompressed(inputStream);
		return rootTag;
	}

	public static TagCompound readFromInternalFile(String fileName) throws Exception {
		FileHandle file = Gdx.files.internal(fileName);

		InputStream inputStream = file.read();
		return readCompressed(inputStream);
	}

	public static void writeToFile(TagCompound rootTag, File file) throws Exception {
		if (file.exists()) {
			file.delete();
		}

		FileOutputStream outputStream = new FileOutputStream(file);
		writeCompressed(outputStream, rootTag);
		outputStream.flush();
		outputStream.close();
	}

}
