package de.howaner.Poketherus.screens;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.Screen;
import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.GL20;
import com.badlogic.gdx.graphics.OrthographicCamera;
import com.badlogic.gdx.graphics.g2d.BitmapFont;
import com.badlogic.gdx.graphics.g2d.BitmapFont.TextBounds;
import com.badlogic.gdx.graphics.g2d.SpriteBatch;
import de.howaner.Poketherus.PokemonGame;
import de.howaner.Poketherus.multiplayer.NetworkManager;
import java.net.InetAddress;

public class ConnectScreen implements Screen {
	private OrthographicCamera camera;
	private SpriteBatch spriteBatch;
	private BitmapFont font;
	private String currentText = "";
	private Thread connectingThread;

	private final String username, password;

	public ConnectScreen(String username, String password) {
		this.username = username;
		this.password = password;
	}

	public void startConnecting() {
		this.connectingThread = new Thread("Server Connector") {
			@Override
			public void run() {
				try {
					ConnectScreen.this.setCurrentText("Erstelle Netzwerkmanager");

					// DEBUG:
					InetAddress address = InetAddress.getByName("howaner.de");
					int port = 1337;

					NetworkManager netManager = NetworkManager.createNetworkManager(address, port);
					netManager.getClientHandle().setConnectScreen(ConnectScreen.this);
					PokemonGame.getGameInstance().setMultiplayerHandler(netManager.getClientHandle());

					ConnectScreen.this.setCurrentText("Verbinden");
					Gdx.app.log("Multiplayer", "Connecting to server ...");
					netManager.getClientHandle().setLoginDetails(ConnectScreen.this.username, ConnectScreen.this.password);
					netManager.getClientHandle().connectToServer();
				} catch (final Exception ex) {
					Gdx.app.error("Multiplayer", "Can't connect to server", ex);
					PokemonGame.getGameInstance().setMultiplayerHandler(null);
					Gdx.app.postRunnable(new Runnable() {
						@Override
						public void run() {
							PokemonGame.getGameInstance().setScreen(new DisconnectScreen("Verbindung fehlgeschlagen:\n" + ex.getMessage()));
						}
					});
				}
			}
		};
		this.connectingThread.start();
	}

	@Override
	public void show() {
		this.camera = new OrthographicCamera();
		this.spriteBatch = new SpriteBatch();

		this.font = new BitmapFont();
		this.font.setColor(Color.WHITE);
		this.font.setScale(2f);

		this.startConnecting();
	}

	@Override
	public void render(float delta) {
		Gdx.gl20.glClearColor(0, 0, 0, 1);
		Gdx.gl20.glClear(GL20.GL_COLOR_BUFFER_BIT);

		this.spriteBatch.setProjectionMatrix(this.camera.combined);
		this.spriteBatch.begin();
		this.drawTitle("Verbinde zu Server ...");
		this.drawStatus(this.currentText);
		this.spriteBatch.end();
	}

	private void drawTitle(String text) {
		TextBounds bounds = this.font.getBounds(text);

		float x = this.camera.viewportWidth / 2 - bounds.width / 2;
		float y = this.camera.viewportHeight - bounds.height - 20;

		this.font.draw(this.spriteBatch, text, x, y);
	}

	private void drawStatus(String text) {
		TextBounds bounds = this.font.getMultiLineBounds(text);

		float y = this.camera.viewportHeight / 2 - bounds.height / 2;
		this.font.drawMultiLine(this.spriteBatch, text, 0, y, this.camera.viewportWidth, BitmapFont.HAlignment.CENTER);
	}

	@Override
	public void resize(int width, int height) {
		this.camera.setToOrtho(false, width, height);
		this.camera.update();
	}

	public void setCurrentText(String text) {
		this.currentText = text;
	}

	@Override
	public void hide() {
		this.dispose();
	}

	@Override
	public void pause() {
		
	}

	@Override
	public void resume() {
		
	}

	@Override
	public void dispose() {
		this.spriteBatch.dispose();
		this.font.dispose();
		if (this.connectingThread.isAlive())
			this.connectingThread.interrupt();

		if (PokemonGame.getGameInstance().getMultiplayerHandler() != null) {
			PokemonGame.getGameInstance().getMultiplayerHandler().setConnectScreen(null);
		}
	}

}
