package de.howaner.Poketherus.screens;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.Screen;
import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.GL20;
import com.badlogic.gdx.graphics.OrthographicCamera;
import com.badlogic.gdx.graphics.g2d.BitmapFont;
import com.badlogic.gdx.graphics.g2d.BitmapFont.TextBounds;
import com.badlogic.gdx.graphics.g2d.SpriteBatch;

public class DisconnectScreen implements Screen {
	private OrthographicCamera camera;
	private SpriteBatch spriteBatch;
	private BitmapFont font;
	private String text = "";

	public DisconnectScreen(String text) {
		this.text = text;
	}

	@Override
	public void show() {
		this.camera = new OrthographicCamera();
		this.spriteBatch = new SpriteBatch();

		this.font = new BitmapFont();
		this.font.setColor(Color.WHITE);
		this.font.setScale(2f);
	}

	@Override
	public void render(float delta) {
		Gdx.gl20.glClearColor(0, 0, 0, 1);
		Gdx.gl20.glClear(GL20.GL_COLOR_BUFFER_BIT);

		this.spriteBatch.setProjectionMatrix(this.camera.combined);
		this.spriteBatch.begin();
		this.drawTitle("Verbindung getrennt");
		this.drawMessage(this.text);
		this.spriteBatch.end();
	}

	private void drawTitle(String text) {
		TextBounds bounds = this.font.getBounds(text);

		float x = this.camera.viewportWidth / 2 - bounds.width / 2;
		float y = this.camera.viewportHeight - bounds.height - 20;

		this.font.draw(this.spriteBatch, text, x, y);
	}

	private void drawMessage(String text) {
		TextBounds bounds = this.font.getMultiLineBounds(text);

		float y = this.camera.viewportHeight / 2 - bounds.height / 2;
		this.font.drawMultiLine(this.spriteBatch, text, 0, y, this.camera.viewportWidth, BitmapFont.HAlignment.CENTER);
	}

	@Override
	public void resize(int width, int height) {
		this.camera.setToOrtho(false, width, height);
		this.camera.update();
	}

	public String getText() {
		return this.text;
	}

	public void setText(String text) {
		this.text = text;
	}

	@Override
	public void hide() {
		this.dispose();
	}

	@Override
	public void pause() {
		
	}

	@Override
	public void resume() {
		
	}

	@Override
	public void dispose() {
		this.spriteBatch.dispose();
		this.font.dispose();
	}

}
