package de.howaner.Poketherus.screens;

import com.badlogic.gdx.Application;
import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.Screen;
import com.badlogic.gdx.graphics.GL20;
import com.badlogic.gdx.graphics.g2d.SpriteBatch;
import com.badlogic.gdx.graphics.g2d.TextureRegion;
import com.badlogic.gdx.scenes.scene2d.InputEvent;
import com.badlogic.gdx.scenes.scene2d.InputListener;
import com.badlogic.gdx.scenes.scene2d.Stage;
import com.badlogic.gdx.scenes.scene2d.ui.Image;
import com.badlogic.gdx.utils.Scaling;
import com.badlogic.gdx.utils.viewport.ScalingViewport;
import com.badlogic.gdx.utils.viewport.ScreenViewport;
import de.howaner.Poketherus.PokemonGame;
import de.howaner.Poketherus.gui.Button;
import de.howaner.Poketherus.gui.CheckButton;
import de.howaner.Poketherus.gui.TextFieldWithLabel;
import de.howaner.Poketherus.multiplayer.util.CryptManager;
import de.howaner.Poketherus.resources.TextureManager;

public class MainMenuScreen implements Screen {
	private Stage stage;
	private SpriteBatch batch;
	private Stage backgroundStage;
	private TextureRegion box;

	// Other:
	private ButtonListener buttonListener;

	// Textboxes:
	private TextFieldWithLabel nameField;
	private TextFieldWithLabel passwordField;

	public MainMenuScreen() {
		this.buttonListener = new ButtonListener();
	}

	@Override
	public void render(float delta) {
		Gdx.gl20.glClearColor(0, 0, 0, 1);
		Gdx.gl20.glClear(GL20.GL_COLOR_BUFFER_BIT);

		this.stage.act();
		this.backgroundStage.act();

		this.batch.begin();
		this.backgroundStage.draw();
		this.stage.draw();
		this.batch.end();
	}

	@Override
	public void resize(int width, int height) {
		this.backgroundStage.getViewport().update(width, height);
		this.stage.getViewport().update(width, height);
	}

	@Override
	public void show() {
		this.batch = new SpriteBatch();

		TextureRegion background = new TextureRegion(TextureManager.MAIN_MENU_BACKGROUND);
		this.backgroundStage = new Stage(new ScalingViewport(Scaling.fit, background.getRegionWidth(), background.getRegionHeight()));
		this.backgroundStage.addActor(new Image(background));

		this.box = new TextureRegion(TextureManager.MAIN_MENU_BOX);
		if (Gdx.app.getType() == Application.ApplicationType.Android) {
			this.stage = new Stage(new ScalingViewport(Scaling.stretch, box.getRegionWidth(), box.getRegionHeight()));
		} else {
			this.stage = new Stage(new ScreenViewport());
		}
		Gdx.input.setInputProcessor(this.stage);

		Image boxImage = new Image(box);
		boxImage.setSize(box.getRegionWidth(), box.getRegionHeight());
		boxImage.setPosition(this.stage.getWidth() / 2 - (box.getRegionWidth() - 79) / 2, this.stage.getHeight() / 2 - box.getRegionHeight() / 2);
		this.stage.addActor(boxImage);

		this.nameField = new TextFieldWithLabel(0f, 0f, "USERNAME:");
		nameField.setLabelWidth(160);
		nameField.setCenterPosition(this.stage.getWidth() / 2, this.stage.getHeight() / 2 - 10f);
		nameField.getTextField().setMaxLength(16);
		nameField.addToStage(this.stage);

		this.passwordField = new TextFieldWithLabel(nameField.getX(), nameField.getY() - 50, "PASSWORD:");
		passwordField.setLabelWidth(160);
		passwordField.getTextField().setPasswordCharacter('*');
		passwordField.getTextField().setPasswordMode(true);
		passwordField.getTextField().setMaxLength(32);
		passwordField.addToStage(this.stage);

		CheckButton rememberButton = new CheckButton("Remember?");
		rememberButton.setPosition(passwordField.getX(), passwordField.getY() - 50f);
		this.stage.addActor(rememberButton);

		Button loginButton = new Button("LOGIN");
		loginButton.setPosition(rememberButton.getX() - 5f, boxImage.getY() - boxImage.getImageHeight() + loginButton.getHeight());
		loginButton.addListener(this.buttonListener);
		this.stage.addActor(loginButton);

		Button optionsButton = new Button("OPTIONS");
		optionsButton.setPosition(loginButton.getX() + loginButton.getWidth() + 20f, loginButton.getY());
		this.stage.addActor(optionsButton);

		Button exitButton = new Button("EXIT");
		exitButton.setPosition(optionsButton.getX() + optionsButton.getWidth() + 20f, optionsButton.getY());
		this.stage.addActor(exitButton);
	}

	@Override
	public void hide() {
		this.dispose();
	}

	@Override
	public void pause() {

	}

	@Override
	public void resume() {

	}

	@Override
	public void dispose() {
		this.batch.dispose();
		this.stage.dispose();
		this.backgroundStage.dispose();
	}

	public class ButtonListener extends InputListener {

		@Override
		public boolean touchDown(InputEvent event, float x, float y, int pointer, int button) {
			Gdx.app.log("Sd", "Hash: " + CryptManager.createMD5(MainMenuScreen.this.passwordField.getTextField().getText()));
			PokemonGame.getGameInstance().setScreen(new ConnectScreen(MainMenuScreen.this.nameField.getTextField().getText(), MainMenuScreen.this.passwordField.getTextField().getText()));
			return false;
		}

		public void touchUp(InputEvent event, float x, float y, int pointer, int button) {
			Gdx.app.log("my app", "Released");
		}
	}

}
