package de.howaner.Poketherus.screens;

import aurelienribon.tweenengine.Tween;
import aurelienribon.tweenengine.TweenManager;
import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.Screen;
import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.GL20;
import com.badlogic.gdx.graphics.OrthographicCamera;
import com.badlogic.gdx.graphics.Texture;
import com.badlogic.gdx.graphics.g2d.BitmapFont;
import com.badlogic.gdx.graphics.g2d.Sprite;
import com.badlogic.gdx.graphics.g2d.SpriteBatch;
import com.badlogic.gdx.math.Vector2;
import de.howaner.Poketherus.PokemonGame;
import de.howaner.Poketherus.entity.CameraPlayer;
import de.howaner.Poketherus.input.MainMenuInput;
import de.howaner.Poketherus.screens.util.MainMenuButton;
import de.howaner.Poketherus.tween.SpriteAccessor;
import java.util.ArrayList;
import java.util.List;

public class OldMainMenuScreen implements Screen {
	private OrthographicCamera camera;
	private SpriteBatch sb;
	private TweenManager tweenManager;

	private Sprite backgroundSprite;
	private Texture backgroundTexture;

	public Sprite blackSprite;
	private Texture blackTexture;

	private Texture buttonTexture;
	private BitmapFont buttonFont;
	public List<MainMenuButton> buttons = new ArrayList<MainMenuButton>();

	private MainMenuButton currentHoverButton = null;
	private Texture buttonHoverTexture;
	private Sprite buttonHoverSprite;

	private final int[][] pictureScales = new int[][] {
		{480, 272},
		{640, 360},
		{848, 480},
		{852, 480},
		{858, 484},
		{960, 540},
		{964, 544},
		{1024, 576},
		{1072, 600},
		{1280, 720},
		{1360, 768},
		{1366, 768},
		{1376, 768},
		{1600, 900},
		{1920, 1080},
		{2048, 1152},
		{2560, 1440},
		{3200, 1600},
		{3840, 2160},
	};

	@Override
	public void render(float delta) {
		Gdx.gl20.glClearColor(0, 0, 0, 1);
		Gdx.gl20.glClear(GL20.GL_COLOR_BUFFER_BIT);

		this.sb.setProjectionMatrix(this.camera.combined);
		this.sb.begin();
		this.backgroundSprite.draw(this.sb);
		this.blackSprite.draw(this.sb);

		for (MainMenuButton button : this.buttons) {
			button.draw(this.sb);
		}

		if (this.currentHoverButton != null) {
			this.buttonHoverSprite.draw(this.sb);
		}
		this.sb.end();

		this.tweenManager.update(delta * 3f);
	}

	public Vector2 getBestScale(int renderWidth, int renderHeight) {
		int width = this.pictureScales[0][0];
		int height = this.pictureScales[0][1];
		for (int[] scale : this.pictureScales) {
			width = scale[0];
			height = scale[1];
			if (renderWidth <= scale[0] && renderHeight <= scale[1]) {
				break;
			}
		}

		return new Vector2(width, height);
	}

	@Override
	public void resize(int width, int height) {
		Vector2 bestScale = this.getBestScale(width, height);
		this.backgroundSprite.setSize(bestScale.x, bestScale.y);
		this.blackSprite.setCenter(width / 2, height / 2);

		for (int i = 0; i < this.buttons.size(); i++) {
			MainMenuButton button = this.buttons.get(i);
			button.setCenterX(this.blackSprite.getX() + this.blackSprite.getWidth() / 2);
			button.setY(height - (this.blackSprite.getY() + 95 + (this.buttonTexture.getHeight() + 20) * i));
		}

		this.camera.setToOrtho(false, width, height);
		this.camera.update();
	}

	@Override
	public void show() {
		this.camera = new OrthographicCamera(1920, 1080);
		this.camera.position.set(this.camera.viewportWidth * .5f, this.camera.viewportHeight * .5f, 0f);
		this.camera.update();
		this.sb = new SpriteBatch();

		this.backgroundTexture = new Texture(Gdx.files.internal("main-menu/background.png"));
		this.backgroundTexture.setFilter(Texture.TextureFilter.Linear, Texture.TextureFilter.Linear);
		this.backgroundSprite = new Sprite(this.backgroundTexture);

		this.blackTexture = new Texture(Gdx.files.internal("main-menu/black.png"));
		this.blackTexture.setFilter(Texture.TextureFilter.Linear, Texture.TextureFilter.Linear);
		this.blackSprite = new Sprite(this.blackTexture);
		this.blackSprite.setSize(this.blackTexture.getWidth(), this.blackTexture.getHeight());

		this.buttonTexture = new Texture(Gdx.files.internal("main-menu/button.png"));
		this.buttonTexture.setFilter(Texture.TextureFilter.Linear, Texture.TextureFilter.Linear);
		this.buttonHoverTexture = new Texture(Gdx.files.internal("main-menu/button-hover.png"));
		this.buttonHoverTexture.setFilter(Texture.TextureFilter.Linear, Texture.TextureFilter.Linear);
		this.buttonHoverSprite = new Sprite(this.buttonHoverTexture);
		this.buttonHoverSprite.setSize(this.buttonHoverTexture.getWidth(), this.buttonHoverTexture.getHeight());

		this.addButtons();

		Gdx.input.setInputProcessor(new MainMenuInput(this));

		this.tweenManager = new TweenManager();
		Tween.registerAccessor(Sprite.class, new SpriteAccessor());
	}

	private void addButtons() {
		this.buttonFont = new BitmapFont();
		this.buttonFont.setColor(Color.BLACK);
		this.buttonFont.setScale(2f);

		MainMenuButton singlePlayer = new MainMenuButton("Singleplayer", this.buttonFont, this.buttonTexture);
		MainMenuButton multiPlayer = new MainMenuButton("Multiplayer", this.buttonFont, this.buttonTexture);
		MainMenuButton options = new MainMenuButton("Optionen", this.buttonFont, this.buttonTexture);
		MainMenuButton exit = new MainMenuButton("Beenden", this.buttonFont, this.buttonTexture);

		this.buttons.add(singlePlayer);
		this.buttons.add(multiPlayer);
		this.buttons.add(options);
		this.buttons.add(exit);
	}

	public void setHoverButton(MainMenuButton button) {
		if (this.currentHoverButton == button) {
			return;
		}
		this.tweenManager.killAll();

		this.currentHoverButton = button;
		if (button != null) {
			this.buttonHoverSprite.setPosition(button.getX() - 3, button.getY() - 3);
			Tween.set(this.buttonHoverSprite, 0).target(0).start(this.tweenManager);
			Tween.to(this.buttonHoverSprite, 0, 1.0f).target(1).repeatYoyo(100, .2f).start(this.tweenManager);
			this.tweenManager.update(0.4f);
		}
	}

	public void clickButton(MainMenuButton button) {
		if (button.id == 0) {
			CameraPlayer player = new CameraPlayer(0, "Olaf");
			player.setPosition(296 * 16, 102 * 16);

			GameScreen gameScreen = new GameScreen(player);
			gameScreen.loadMainWorld();
			gameScreen.switchToWorld("Hauptwelt");
			gameScreen.updateChunks(false);
			PokemonGame.getGameInstance().setScreen(gameScreen);
		} else if (button.id == 1) {
			Gdx.app.log("Multiplayer", "Clicked the multiplayer button!");
			//PokemonGame.getGameInstance().setScreen(new ConnectScreen());
		} else if (button.id == 2) {
			PokemonGame.getGameInstance().setScreen(new MainMenuScreen());
		} else if (button.id == 3) {
			Gdx.app.exit();
		}
	}

	public MainMenuButton findMenuButton(int x, int y) {
		y = Gdx.graphics.getHeight() - 1 - y;
		for (MainMenuButton button : this.buttons) {
			if (x >= button.getX() && x <= button.getX() + button.getWidth() && y >= button.getY() && y <= button.getY() + button.getHeight()) {
				return button;
			}
		}
		return null;
	}

	@Override
	public void hide() {
		this.dispose();
	}

	@Override
	public void pause() {
		
	}

	@Override
	public void resume() {
		
	}

	@Override
	public void dispose() {
		Gdx.input.setInputProcessor(null);
		this.sb.dispose();
		this.tweenManager.killAll();
		this.backgroundTexture.dispose();
		this.blackTexture.dispose();
		this.buttonTexture.dispose();
		this.buttonFont.dispose();
		this.buttons.clear();
		this.buttonHoverTexture.dispose();
	}

}
