package de.howaner.Poketherus.screens;

import aurelienribon.tweenengine.BaseTween;
import aurelienribon.tweenengine.Tween;
import aurelienribon.tweenengine.TweenCallback;
import aurelienribon.tweenengine.TweenManager;
import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.Screen;
import com.badlogic.gdx.graphics.GL20;
import com.badlogic.gdx.graphics.Texture;
import com.badlogic.gdx.graphics.g2d.Sprite;
import com.badlogic.gdx.graphics.g2d.SpriteBatch;
import de.howaner.Poketherus.PokemonGame;
import de.howaner.Poketherus.input.SplashInput;
import de.howaner.Poketherus.tween.SpriteAccessor;

public class SplashScreen implements Screen {
	private SpriteBatch batch;
	private Sprite splash;
	private TweenManager tweenManager;

	@Override
	public void render(float delta) {
		Gdx.gl20.glClearColor(0, 0, 0, 1);
		Gdx.gl20.glClear(GL20.GL_COLOR_BUFFER_BIT);

		this.batch.begin();
		this.splash.draw(this.batch);
		this.batch.end();

		this.tweenManager.update(delta);
	}

	@Override
	public void resize(int width, int height) {
		
	}

	@Override
	public void show() {
		this.batch = new SpriteBatch();

		// Initialize tween
		this.tweenManager = new TweenManager();
		Tween.registerAccessor(Sprite.class, new SpriteAccessor());

		this.splash = new Sprite(new Texture("splash.png"));
		float ratio = this.splash.getHeight() / this.splash.getWidth();
		this.splash.setSize(Gdx.graphics.getWidth(), Gdx.graphics.getWidth() * ratio);
		this.splash.setY(Gdx.graphics.getHeight() / 2 - this.splash.getHeight() / 2);

		Tween.set(this.splash, SpriteAccessor.ALPHA).target(0).start(this.tweenManager);
		Tween.to(this.splash, SpriteAccessor.ALPHA, 1.5f).target(1).repeatYoyo(1, 0.5f).setCallback(new TweenCallback() {
			@Override
			public void onEvent(int type, BaseTween<?> source) {
				PokemonGame.getGameInstance().setScreen(new OldMainMenuScreen());
			}
		}).start(this.tweenManager);

		Gdx.input.setInputProcessor(new SplashInput(this));
	}

	public void skipSplashScreen() {
		this.tweenManager.killAll();

		Tween.set(this.splash, SpriteAccessor.ALPHA).target(1).setCallback(new TweenCallback() {
			@Override
			public void onEvent(int type, BaseTween<?> source) {
				PokemonGame.getGameInstance().setScreen(new OldMainMenuScreen());
			}
		}).start(this.tweenManager);
	}

	@Override
	public void hide() {
		this.dispose();
	}

	@Override
	public void pause() {
		
	}

	@Override
	public void resume() {
		
	}

	@Override
	public void dispose() {
		Gdx.input.setInputProcessor(null);
		this.batch.dispose();
		this.splash.getTexture().dispose();
	}

}
